@@ -11,6 +11,9 @@ test_that("getDim matches whole strings", {
   expect_identical(getDim("bird", animal,  dimCode = TRUE), 3.2)
   expect_identical(getDim("april", animal,  dimCode = FALSE), "month")
   expect_identical(getDim("april", animal,  dimCode = TRUE), 2.2)
+  expect_identical(getDim(c("AFR","CPA"), pop,  fullmatch = TRUE), numeric(0))
+  expect_identical(getDim(getRegions(pop), pop,  fullmatch = TRUE), 1.1)
+  
 })
 
 
